indexing
	description: "A controlled part of a control system"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	CONTROL_OBJECT
inherit
	SIMULATED

feature -- Access
	predicate_count: INTEGER is
			-- Number of predicates
		deferred
		end

	action_count: INTEGER is
			-- Number of actions
		deferred
		end

feature -- Basic operations
	predicate_values: BIT_VECTOR is
			-- Bit vector with values of all the predicates
		local
			i: INTEGER
		do
			create Result.make(1, predicate_count)
			from
				i := Result.lower
			until
				i > Result.upper
			loop
				Result[i] := x(i)
				i := i + 1
			end
		ensure
			result_size_correct: Result.count = predicate_count
		end

	do_actions(actions: BIT_VECTOR) is
			-- Execute actions marked `True' in `actions'
		require
			actions_size_correct: actions /= Void and then actions.count = action_count
		local
			i: INTEGER
		do
			from
				i := actions.lower
			until
				i > actions.upper
			loop
				if (actions[i]) then z(i) end
				i := i + 1
			end
		end

feature -- Output
	show_computational_state is
			-- Print or draw computational state of `Current'
			-- Do nothing by default, should be redefined if emulation visualization is needed
		do
		end

feature {NONE} -- Implementation
	x (i: INTEGER): BOOLEAN is
			-- Value of predicate with index `i'
			-- Descendants should assign indices to predicates
			-- and call proper one here
		require
			i_not_too_small: i >= 1
			i_not_too_large: i <= predicate_count
		deferred
		end

	z (i: INTEGER) is
			-- Execute action with index `i'
			-- Descendants should assign indices to actions
			-- and call proper one here
		require
			i_not_too_small: i >= 1
			i_not_too_large: i <= action_count
		deferred
		end
end
