indexing
	description: "Evolving Mealy automata"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	GENETIC_AUTO

inherit
	GENETIC_CODE

	AUTO
	redefine
		states, make
	end
create
	make

feature -- Initialization
	make (prototype: GENETIC_STATE) is
			-- Make automaton with states that are clones of `prototype'
		do
			Precursor(prototype)
		end

	randomize is
			-- Randomize transition and output functions of `Current'
		local
			i: INTEGER
		do
			from
				i := states.lower
			until
				i > states.upper
			loop
				states[i].randomize
				i := i + 1
			end
		end

feature -- Access
	mutation_index: REAL is
			-- Mutation index of each state
		do
			Result := states[current_state].mutation_index
		end

feature -- Status report
	is_compatible_with (other: like Current): BOOLEAN is
			-- Can `Current' and `other' have chilren?
		do
			Result := states[states.lower].is_compatible_with(other.states[other.states.lower])
		end

feature -- Basic operations
	mutant: like Current is
			-- A mutated version of `Current'
		local
			i: INTEGER
		do
			Result := deep_twin
			from
				i := states.lower
			until
				i > states.upper
			loop
				Result.states[i] := states[i].mutant
				i := i + 1
			end
		end

	children (other_parent: like Current) : ARRAY [like Current] is
			-- A pair of children who's parent are `Current' and `other_parent'
		local
			i: INTEGER
			states_children: ARRAY[GENETIC_STATE]
		do
			create Result.make(1, 2)
			Result[1] := deep_twin
			Result[2] := deep_twin
			from
				i := states.lower
			until
				i > states.upper
			loop
				states_children := states[i].children (other_parent.states[i])
				Result[1].states[i] := states_children[1]
				Result[2].states[i] := states_children[2]
				i := i + 1
			end
		end

	set_mutation_index (p:REAL) is
			-- Set mutation index of each state
		local
			i: INTEGER
		do
			from
				i := states.lower
			until
				i > states.upper
			loop
				states[i].set_mutation_index(p)
				i := i + 1
			end
		end

	to_file (fn: STRING) is
		local
			f: PLAIN_TEXT_FILE
			st: GENETIC_REDUCED_TABLE_STATE
			i: INTEGER
		do
			create f.make_open_write (fn)
			from i := states.lower
			until i > states.upper
			loop
				st ?= states.item (i)
				f.put_string (st.to_str)
				f.put_new_line
				i := i + 1
			end
			f.close
		end

	from_file (fn: STRING) is
		local
			f: PLAIN_TEXT_FILE
			st: GENETIC_REDUCED_TABLE_STATE
			i: INTEGER
		do
			create f.make_open_read (fn)
			from i := states.lower
			until i > states.upper
			loop
				st ?= states.item (i)
				st.from_file (f)
				i := i + 1
			end
			f.close
		end


feature {GENETIC_AUTO} -- Implementation
	states: ARRAY[GENETIC_STATE]
			-- An array of automaton's states	
invariant
	-- mutaion_index_the_same: forall i in [1..state_count] states[i].mutation_index = mutation_index
end
