indexing
	description: "Represents portions of genetic code that can mutate and cross with each other throughout evolution process"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	GENETIC_CODE
inherit
	STORABLE

feature -- Initialization
	randomize is
			-- Make `Current' random
		deferred
		end

feature -- Access
	mutation_index: REAL is
			-- Number proportional to probability that `mutant' /= `Current'
			-- ToDo: may be, remove this feature? Leave only in "leaf" genetic classes
		deferred
		end

feature -- Status report
	is_compatible_with (other: like Current): BOOLEAN is
			-- Can `Current' and `other' have chilren?
		require
			other_exists: other /= Void
		deferred
		ensure
			equal_implies_compatible: is_deep_equal(other) implies Result
			symmetric: Result implies (other.is_compatible_with(Current))
		end

feature -- Basic operations
	mutant: like Current is
			-- A mutated version of `Current'
		deferred
		ensure
			mutant_exists: Result /= Void
		end

	children (other_parent: like Current) : ARRAY [like Current] is
			-- A pair of children who's parent are `Current' and `other_parent'
		require
			other_parent_exists: other_parent /= Void
			parents_compatible: is_compatible_with (other_parent)
		deferred
		ensure
			children_exists: Result /= Void
			two_children: Result.count = 2
		end

	set_mutation_index(p: REAL) is
			-- Make `mutation_index' equal to `p'
		require
			p_not_negative: p >= 0
			p_not_too_large: p <= 1
		deferred
		ensure
			mutation_index_set: mutation_index = p
		end
invariant
	mutation_index_not_negative: mutation_index >= 0
	mutation_index_not_too_large: mutation_index <= 1

	-- transitive_compatible: forall a, b: GENETIC_CODE
		-- Current.is_compatible_with(a) and a.is_compatible_with(b) imples Current.is_compatible_with(b)
--	mutant_compatible: is_compatible_with(mutant)
	-- children_compatible: forall a: GENETIC_CODE
		-- is_compatible_with(a) imples (is_compatible_with(children(a)[1]) and is_compatible_with(children(a)[2]))
end
