indexing
	description: "Evolving Mealy automata states represented by full tables"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	GENETIC_FULL_TABLE_STATE

inherit
	GENETIC_STATE

	FULL_TABLE_STATE
	export
		{GENETIC_FULL_TABLE_STATE} transition_function, output_function
	redefine
		make
	end
create
	make, make_compatible

feature --Initialization
	make (p, a, s: INTEGER) is
			-- Create state with `p' predicates and `a' actions
		do
			Precursor(p, a, s)
			mutation_index := default_mutation_index
		ensure then
			mutation_index_default: mutation_index = default_mutation_index
		end

	randomize is
			-- Make transition and output functions random
		local
			i: INTEGER
		do
			from
				i := transition_function.lower
			until
				i > transition_function.upper
			loop
				r.forth
				transition_function[i] := r.bounded_item(1, state_count)
				r.forth
				output_function[i].as_integer(r.bounded_item(1, 1 |<< action_count - 1))
				i := i + 1
			end
		end

feature -- Access
	mutation_index: REAL
			-- Number proportional to probability that `mutant' /= `Current'	

feature -- Basic operations
	mutant: like Current is
			-- A mutated version of `Current'
		local
			i, actions_present, k: INTEGER
			o: BIT_VECTOR
		do
			Result := deep_twin
			from
				i := Result.transition_function.lower
			until
				i > Result.transition_function.upper
			loop
--				r.forth
--				if r.decide(mutation_index) then
--						-- Mutate target state
--					r.forth
--					Result.transition_function[i] := r.bounded_item (1, state_count)
--				end
--				r.forth
--				if r.decide(mutation_index) then
--						-- Mutate output
--					o := Result.output_function[i]
--					actions_present := o.occurrences(True)
--					if (actions_present = action_count) or (actions_present = 0) then
--						r.forth
--						k := r.bounded_item (1, action_count)
--						o[k] := not o[k]
--					else
--						from k := 1 until k > action_count loop
--							r.forth
--							o[k] := (r.bounded_item (1, action_count) <= actions_present)
--							k := k + 1
--						end
--					end
--				end

				o := Result.output_function[i]
				from k := 1 until k > action_count loop
					r.forth
					if r.decide(0.001) then
						o[k] := not o[k]
					end
					k := k + 1
				end

				i := i + 1
			end

		end

	children (other_parent: like Current) : ARRAY [like Current] is
			-- A pair of children who's parent are `Current' and `other_parent'
		local
			first_child, second_child: like Current
			cross_point: INTEGER
			trans_temp: ARRAY[INTEGER]
			out_temp: ARRAY[BIT_VECTOR]
		do
			first_child := deep_twin
			second_child := other_parent.deep_twin

			r.forth
			cross_point := r.bounded_item(transition_function.lower + 1, transition_function.upper)

				-- Swapping transition functions' tails
			trans_temp := first_child.transition_function.subarray(cross_point, transition_function.upper)
			first_child.transition_function.subcopy(second_child.transition_function, cross_point, transition_function.upper, cross_point)
			second_child.transition_function.subcopy(trans_temp, cross_point, transition_function.upper, cross_point)

				-- Swapping output functions' tails
			out_temp := first_child.output_function.subarray(cross_point, output_function.upper)
			first_child.output_function.subcopy(second_child.output_function, cross_point, output_function.upper, cross_point)
			second_child.output_function.subcopy(out_temp, cross_point, output_function.upper, cross_point)

			create Result.make(1, 2)
			Result[1] := first_child
			Result[2] := second_child

		end

	set_mutation_index(p: REAL) is
			-- Make `mutation_index' equal to `p'
		do
			mutation_index := p
		end

feature {NONE} -- Implementation
	default_mutation_index: REAL is 0.01
			-- Default value of `mutation_index'

	r: MY_RANDOM is
			-- Random generator for implementation needs
		once
			create Result.make
		end

invariant
	r_exists: r /= Void
end
