indexing
	description: "Evolving Mealy automata states represented by reduced tables"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	GENETIC_REDUCED_TABLE_STATE

inherit
	GENETIC_FULL_TABLE_STATE
		rename
			predicate_count as significant_predicate_count,
			current_input as significant_input,
			make as make_full,
			make_compatible as make_compatible_full
		export {NONE}
			make_full, make_compatible_full
		undefine
			set_input,
			update_input_index
		redefine
			randomize, mutant, children,
			significant_predicate_count, significant_input  -- To avoid conflict
		end

	REDUCED_TABLE_STATE
		export
			{GENETIC_REDUCED_TABLE_STATE} transition_function, output_function
		undefine
			make_full, make_compatible_full
		redefine
			make,
			significant_predicate_count, significant_input -- To avoid conflict
		select
			predicate_count, current_input
		end
create
	make, make_compatible

feature --Initialization
	make (p, a, s, sp: INTEGER) is
			-- Create state with `p' predicates, of which `sp' are significant,
			-- `a' actions and capable of connecting to `s' other states
		do
			Precursor(p, a, s, sp)
		ensure then
			mutation_index_default: mutation_index = default_mutation_index
		end

	randomize is
			-- Make significant predicates set, transition and output functions random
		local
			i, inserted: INTEGER
		do
			significant_predicates.clear_all
			-- Seting random significant predicates			
			from
				inserted := 0
			until
				inserted = significant_predicate_count
			loop
				r.forth
				i := r.bounded_item (significant_predicates.lower, significant_predicates.upper)
				if not significant_predicates [i] then
					significant_predicates [i] := True
					inserted := inserted + 1
				end
			end
			Precursor
		end

feature -- Access
	significant_predicate_count: INTEGER

	significant_input: BIT_VECTOR

feature -- Basic operations
	mutant: like Current is
			-- A mutated version of `Current'
		local
			full_mutant: GENETIC_FULL_TABLE_STATE
			i, j: INTEGER
		do
			Result := deep_twin

			full_mutant := Precursor
			Result.set_transition_function (full_mutant.transition_function)
			Result.set_output_function (full_mutant.output_function)

--			r.forth
--			if significant_predicate_count > 0 and significant_predicate_count < predicate_count and r.decide(mutation_index) then
--				-- Choose random significant predicate and save its index to `i'
--				from
--					r.forth
--					i := r.bounded_item (significant_predicates.lower, significant_predicates.upper)
--				until
--					significant_predicates [i]
--				loop
--					r.forth
--					i := r.bounded_item (significant_predicates.lower, significant_predicates.upper)
--				end
--				-- Choose random insignificant predicate and save its index to `j'
--				from
--					r.forth
--					j := r.bounded_item (significant_predicates.lower, significant_predicates.upper)
--				until
--					not significant_predicates [j]
--				loop
--					r.forth
--					j := r.bounded_item (significant_predicates.lower, significant_predicates.upper)
--				end
--				-- Change them
--				Result.significant_predicates [j] := True
--				Result.significant_predicates [i] := False
--			end
		end

	children (other_parent: like Current) : ARRAY [like Current] is
			-- A pair of children who's parents are `Current' and `other_parent'
		local
			i, line: INTEGER
			parents_lines: ARRAY [ARRAY [INTEGER]]
		do
			create Result.make(1, 2)
			Result[1] := deep_twin
			Result[2] := other_parent.deep_twin

			choose_child_significant_predicates (other_parent, Result)

			from
				i := 1
			until
				i > 2
			loop
				from
					line := Result[i].transition_function.lower
				until
					line > Result[i].transition_function.upper
				loop
					parents_lines := parents_line_numbers (line, other_parent, Result[i])
					Result[i].transition_function.put (child_next_state (parents_lines, other_parent), line)
					Result[i].output_function.put (child_actions (parents_lines, other_parent), line)
					line := line + 1
				end
				i := i + 1
			end
		end

	to_str: STRING is
		local
			i: INTEGER
			bv: BIT_VECTOR
		do
			create Result.make_empty
			bv := significant_predicates
			from i := bv.lower
			until i > bv.upper
			loop
				if bv.item (i) then
					Result.append_integer (i)
				end
				i := i + 1
			end
			Result.append (" ")
			Result.append_integer (transition_function.item (transition_function.lower))
			Result.append (" ")
			bv := output_function.item (output_function.lower)
			from i := bv.lower
			until i > bv.upper
			loop
				if bv.item (i) then
					Result.append_integer (1)
				else
					Result.append_integer (0)
				end
				Result.append (" ")
				i := i + 1
			end
			Result.append (" ")
			Result.append (" ")
			Result.append_integer (transition_function.item (transition_function.upper))
			Result.append (" ")
			bv := output_function.item (output_function.upper)
			from i := bv.lower
			until i > bv.upper
			loop
				if bv.item (i) then
					Result.append_integer (1)
				else
					Result.append_integer (0)
				end
				Result.append (" ")
				i := i + 1
			end
		end

	from_file (f: PLAIN_TEXT_FILE) is
		local
			i, j: INTEGER
			bv: BIT_VECTOR
		do
			f.read_integer
			j := f.last_integer
			bv := significant_predicates
			from i := bv.lower
			until i > bv.upper
			loop
				if i = j then
					bv.put (true, i)
				else
					bv.put (false, i)
				end
				i := i + 1
			end
			f.read_integer
			transition_function.put (f.last_integer, transition_function.lower)
			bv := output_function.item (output_function.lower)
			from i := bv.lower
			until i > bv.upper
			loop
				f.read_integer
				bv.put (f.last_integer = 1, i)
				i := i + 1
			end
			f.read_integer
			transition_function.put (f.last_integer, transition_function.upper)
			bv := output_function.item (output_function.upper)
			from i := bv.lower
			until i > bv.upper
			loop
				f.read_integer
				bv.put (f.last_integer = 1, i)
				i := i + 1
			end
		end

feature {NONE} -- Implementation
	choose_child_significant_predicates (other_parent: like Current; ch: ARRAY [like Current]) is
				-- Set significant predicates for children
		require
			other_parent_exists: other_parent /= Void
			children_exists: ch /= Void
			two_children: ch.count = 2
			first_child_exists: ch [1] /= Void
			second_child_exists: ch [2] /= Void
		local
			i, first_child_space, second_child_space: INTEGER
		do
			ch[1].significant_predicates.clear_all
			ch[2].significant_predicates.clear_all

			first_child_space := significant_predicate_count
			second_child_space := significant_predicate_count

			-- Predicates that are significant for both parents should be significant for both children
			from
				i := significant_predicates.lower
			until
				i > significant_predicates.upper
			loop
				if significant_predicates [i] and other_parent.significant_predicates [i] then
					check first_child_space > 0 end
					ch[1].significant_predicates [i] := True
					first_child_space := first_child_space - 1
					check second_child_space > 0 end
					ch[2].significant_predicates [i] := True
					second_child_space := second_child_space - 1
				end
				i := i + 1
			end
			-- Predicates that are significant for only one of parents should be significant for one of children
			-- (to one that has enough space or random if both have space)
			from
				i := significant_predicates.lower
			until
				i > significant_predicates.upper
			loop
				if significant_predicates [i] xor other_parent.significant_predicates [i] then
					if first_child_space > 0 and second_child_space > 0 then
						r.forth
						if r.decide (0.5) then
							ch[1].significant_predicates [i] := True
							first_child_space := first_child_space - 1
						else
							ch[2].significant_predicates [i] := True
							second_child_space := second_child_space - 1
						end
					elseif first_child_space > 0 then
						ch[1].significant_predicates [i] := True
						first_child_space := first_child_space - 1
					else
						check second_child_space > 0 end
						ch[2].significant_predicates [i] := True
						second_child_space := second_child_space - 1
					end
				end
				i := i + 1
			end
		end

	Never_mind: TRINARY is
			-- Trnary `Never_mind' value
		once
			Result := Result.N
		end

	composition (mask: ARRAY [TRINARY]; subvector: BIT_VECTOR): BIT_VECTOR is
			-- Replace undefined positions in `mask' by bits of `subvector'
		require
			mask_exists: mask /= Void
			subvector_exists: subvector /= Void
			subvector_size_correct: subvector.count = mask.occurrences (Never_mind)
		local
			mask_i, subvector_i: INTEGER
		do
			create Result.make (mask.lower, mask.upper)
			from
				mask_i := mask.lower
				subvector_i := subvector.lower
			until
				mask_i > mask.upper
			loop
				if mask [mask_i] = Never_mind then
					Result [mask_i] := subvector [subvector_i]
					subvector_i := subvector_i + 1
				else
					Result [mask_i] := mask [mask_i]
				end
				mask_i := mask_i + 1
			end
		ensure
			result_exists: Result /= Void
			as_large_as_mask: Result.count = mask.count
		end

	line_numbers (mask: ARRAY [TRINARY]): ARRAY [INTEGER] is
			-- Array of lines, corresponding to mask
		require
			mask_exists: mask /= Void
		local
			lines_count, subvector_int, subvector_size: INTEGER
			subvector: BIT_VECTOR
		do
			subvector_size := mask.occurrences (Never_mind)
			create subvector.make (1, subvector_size)

			lines_count := (1 |<< subvector_size)
			create Result.make (1, lines_count)

			from
				subvector_int := 0
			until
				subvector_int = lines_count
			loop
				subvector.as_integer (subvector_int)
				Result [subvector_int + 1] := composition (mask, subvector).to_integer + 1
				subvector_int := subvector_int + 1
			end
		ensure
			result_exists: Result /= Void
			result_size_correct: Result.count = (2 ^ mask.occurrences (Never_mind))
		end


	parents_line_numbers (line: INTEGER; other_parent, child: like Current): ARRAY [ARRAY [INTEGER]] is
				-- Arrays of parent line numbers that correspond to given `line'
				-- Parent line corresponds to `line' if predicates that are siginificant for `child',
				-- have same values in `child' and in parent. If a predicate is significant for `child' and
				-- both parents, then a corresponding line is taken from a single (random) parent
		local
			mask1, mask2: ARRAY [TRINARY]
			i, pred_in_parent1, pred_in_parent2, pred_in_child : INTEGER
			input: BIT_VECTOR
		do
			-- Build masks of aproptiate parent lines
			create mask1.make (1, significant_predicate_count)
			create mask2.make (1, other_parent.significant_predicate_count)

			pred_in_parent1 := 1;
			pred_in_parent2 := 1;
			pred_in_child := 1;

			create input.make (1, child.significant_predicate_count)
			input.as_integer (line - 1)

			from
				i := significant_predicates.lower
			until
				i > significant_predicates.upper
			loop
				if child.significant_predicates [i] then
					-- Put`mask_bit' to one of parents mask (for whom this predicate is significant or random)
--					if significant_predicates [i] then
--						if other_parent.significant_predicates [i] then
--							r.forth
--							if r.decide (0.5) then
--								mask1 [pred_in_parent1] := input[pred_in_child]
--							else
--								mask2 [pred_in_parent2] := input[pred_in_child]
--							end
--						else
--							mask1 [pred_in_parent1] := input[pred_in_child]
--						end
--					else
--						mask2 [pred_in_parent2] := input[pred_in_child]
--					end
					if significant_predicates [i] then
						mask1[pred_in_parent1] := input[pred_in_child]
					end
					if other_parent.significant_predicates[i] then
						mask2[pred_in_parent2] := input[pred_in_child]
					end

					pred_in_child := pred_in_child + 1
				end

				if significant_predicates [i] then
					pred_in_parent1 := pred_in_parent1 + 1
				end
				if other_parent.significant_predicates [i] then
					pred_in_parent2 := pred_in_parent2 + 1
				end
				i := i + 1
			end

			create Result.make(1, 2)
			Result [1] := line_numbers (mask1)
			Result [2] := line_numbers (mask2)
		end

	child_next_state (parent_lines: ARRAY [ARRAY [INTEGER]]; other_parent: like Current): INTEGER is
			-- State that is chosen by voting of `parent_lines'
	local
		target_state: ARRAY [REAL]
		i, state_num: INTEGER
		mul1, mul2: REAL
	do
		create target_state.make (1, state_count)

		mul1 := 0.5 / parent_lines [1].count
		mul2 := 0.5 / parent_lines [2].count
		from
			i := parent_lines [1].lower
		until
			i > parent_lines [1].upper
		loop
			state_num := transition_function [(parent_lines [1]) [i]]
			target_state[state_num] := target_state[state_num] + mul1
			i := i + 1
		end
		from
			i := parent_lines [2].lower
		until
			i > parent_lines [2].upper
		loop
			state_num := other_parent.transition_function [(parent_lines [2]) [i]]
			target_state[state_num] := target_state[state_num] + mul2
			i := i + 1
		end
		r.forth
		Result := r.sample(target_state)
	end

	child_actions (parent_lines: ARRAY [ARRAY [INTEGER]]; other_parent: like Current): BIT_VECTOR is
			-- Actions that are chosen by voting of `parent_lines'
	local
		action_probability: REAL
		i, action_num: INTEGER
		mul1, mul2: REAL
	do
		create Result.make(1, action_count)

		mul1 := 0.5 / parent_lines [1].count
		mul2 := 0.5 / parent_lines [2].count
		from
			action_num := 1
		until
			action_num > action_count
		loop
			action_probability := 0
			from
				i := parent_lines [1].lower
			until
				i > parent_lines [1].upper
			loop
				if (output_function[(parent_lines [1]) [i]]) [action_num] then
					action_probability := action_probability + mul1
				end
				i := i + 1
			end
			from
				i := parent_lines [2].lower
			until
				i > parent_lines [2].upper
			loop
				if (other_parent.output_function[(parent_lines [2]) [i]]) [action_num] then
					action_probability := action_probability + mul2
				end
				i := i + 1
			end
			r.forth
			Result [action_num] := r.decide (action_probability)
			action_num := action_num + 1
		end
	end

end
