indexing
	description: "Objects with inner simulated time"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	SIMULATED

feature -- Access
	elapsed_time: REAL
			-- Simulated time elapsed

	step_duration: REAL
			-- Step duration (in simulated time units)

feature -- Basic operations
	set_step_duration (dt: REAL) is
			-- Set `step_duration' to `dt'
		require
			dt_positive: dt > 0.0
		do
			step_duration := dt
		ensure
			step_duration_set: step_duration = dt
		end

	frozen restart is
			-- Call `actual_restart' and then zero `elapsed_time'
			-- (`actual_restart' should be redefined if some action is needed)
		do
			actual_restart
			elapsed_time := 0.0
		ensure
			time_nulled: elapsed_time = 0.0
		end


	frozen do_step is
			-- Call `actual_do_step' and then increase `elapsed_time'
			-- (`actual_do_step' should be redefined if some action is needed)
		do
			actual_do_step
			elapsed_time := elapsed_time + step_duration
		end

feature {NONE} -- Implementation
	default_step_duration: REAL is
			-- Default step duration [s]
		once
			Result := 1.0
		end

	actual_restart is
			-- Do something before zeroing simulated time in `restart'
		deferred
		end

	actual_do_step is
			-- Do something before increasing simulated time in `do_step'
		deferred
		end
invariant
	time_not_negative: elapsed_time >= 0.0
	step_duration_positive: step_duration > 0.0
end
