class
	X_PLANE_BROKER

create
	make

feature -- Predicates
	moving: BOOLEAN is
			-- Whether plane has signifcant airspeed
		do
			receive
			Result := airspeed_knots >= 5
		end
	can_fly_with_flaps: BOOLEAN is
			-- Whether airspeed is enough not to stall with flaps down
		do
			receive
			Result := airspeed_knots > 70
		end
	can_fly_without_flaps: BOOLEAN is
			-- Whether airspeed is enough not to stall with flaps up
		do
			receive
			Result := airspeed_knots > 100
		end
	flying: BOOLEAN is
			-- Whether plane is not touching the ground
		do
			receive
			Result := altitude_above_ground_ft >= 1
		end
	near_surface: BOOLEAN is
			-- Whether plane is not far from the ground
		do
			receive
			Result := altitude_above_ground_ft < 15
		end
	on_echelon_altitude: BOOLEAN is
			-- Whether plane is on echelon altitude
		do
			receive
			Result := altitude_above_sea_level_ft > 660
		end
	reaching_gps_target: BOOLEAN is
			-- Whether target set in GPS receiver is nearby
		do
			receive
			Result := gps_dist < 0.6
		end

feature -- Actions
	init is
			-- Bring emulator to starting state
		do
			set_to_airport ("KJYO", 0, false)
			half_fill_with_fuel
			set_arrival_nav_freq
			prev_time1 := 0
			prev_time := 0
			defl_int := 0
			time_int := 0
		end
	set_departure_nav_freq is
			-- Set frequency of nav1 receiver to departure ILS
		do
			set_nav1_freq (111.75)
		end
	set_arrival_nav_freq is
			-- Set frequency of nav1 receiver to arrival ILS
		do
			set_nav1_freq (111.30)
		end
	set_gps_to_waypoint_mode is
			-- Set GPS receiver to wapoint mode
		do
			receive
			set_gps_mode_waypoint
		end
	set_flight_director_to_auto is
			-- Set flight director switch to position 3: auto
		do
			receive
			set_fdir_auto
		end
	set_autopilot_localizer_mode is
			-- Push LOC button on autopilot in case it is not already active
		do
			set_autopilot_localizer
		end
	set_autopilot_heading_hold is
		do
			receive
			receive
			if data.item (111).item (2) < 0.5 or data.item (111).item (2) > 1.5 then
				send_char ('y')
			end
			receive
			receive
			if data.item (111).item (2) < 0.5 or data.item (111).item (2) > 1.5 then
				send_char ('y')
			end
		end
	set_autopilot_glide_slope is
		do
			receive
			receive
			if (data.item (111).item (3) < 8.5 or data.item (111).item (3) > 9.5) and data.item (111).item (7) < 0.5 then
				send_char ('o')
			end
			receive
			receive
			if (data.item (111).item (3) < 8.5 or data.item (111).item (3) > 9.5) and data.item (111).item (7) < 0.5 then
				send_char ('o')
			end
		end
	set_autopilot_altitude_hold is
		do
			receive
			send_char ('p')
		end
	set_autopilot_vertical_speed_hold is
		do
			receive
			send_char ('r')
		end
	set_wheel_brake_on is
		do
			receive
			receive
			receive
			if not wheel_brake then
				send_char ('e')
			end
		end
	set_wheel_brake_off is
		do
			receive
			receive
			receive
			if wheel_brake then
				send_char ('e')
			end
		end
	set_throttle_full is
		do
			set_throttle (1)
		end
	set_throttle_half is
		do
			set_throttle (0.5)
		end
	set_throttle_min is
		do
			set_throttle (0)
		end
	set_no_flaps is
		do
			set_flaps (0)
		end
	set_takeoff_flaps is
		do
			set_flaps (0.3)
		end
	set_landing_flaps is
		do
			set_flaps (1)
		end
	set_gear_down is
		do
			set_gear (true)
		end
	set_gear_up is
		do
			set_gear (false)
		end
	set_hsi_to_nav1 is
		local
			i: INTEGER
		do
			receive
			from
				i := 1
			until
				i > 8
			loop
				data.item (103).put (-999, i)
				i := i + 1
			end
			data.item (103).put (0, 4)
			send_one (103)
		end
	set_hsi_to_gps is
		local
			i: INTEGER
		do
			receive
			from
				i := 1
			until
				i > 8
			loop
				data.item (103).put (-999, i)
				i := i + 1
			end
			data.item (103).put (2, 4)
			send_one (103)
		end
	set_vertical_speed_and_altitude is
		local
			i: INTEGER
			speed, alt: REAL
		do
			speed := 100
			alt := 700
			receive
			from
				i := 1
			until
				i > 8
			loop
				data.item (112).put (-999, i)
				i := i + 1
			end
			data.item (112).put (speed, 3)
			data.item (112).put (alt, 4)
			send_one (112)
		end
	translate_autopilot_to_rudder is
			-- Set rudder to the same position as ailerons
		local
			i: INTEGER
		do
			receive
			from
				i := 1
			until
				i > 8
			loop
				data.item (7).put (-999, i)
				i := i + 1
			end
			set_joystick_rudder (data.item (104).item (4) / 30)
			send_one (7)
		end
	set_rudder_centered is
		local
			i: INTEGER
		do
			receive
			from
				i := 1
			until
				i > 8
			loop
				data.item (7).put (-999, i)
				i := i + 1
			end
			set_joystick_rudder (0)
			send_one (7)
		end

feature -- Fitness
	fitness: REAL is
			-- Fitness functon value estimate
		do
			if time_int = 0 then
				Result := 0
			else
				Result := time_int / ((100 + defl_int / time_int) * 0.77)
			end
			if engine_fire then
				Result := Result / 10
			end
			if Result > 1 then
				Result := 1
			end
		end
	old_fitness: REAL is
			-- Fitness functon value estimate
		local
			i, best_i: INTEGER
			cur_dist, least_dist: REAL
		do
			least_dist := 0
			from i := 1
			until i > 10200
			loop
				cur_dist := (plane_x - rt.item (i).pos.x_value) ^ 2 + (plane_y - rt.item (i).pos.y_value) ^ 2 + (plane_z - rt.item (i).pos.z_value) ^ 2
				if least_dist = 0 or cur_dist < least_dist then
					least_dist := cur_dist
					best_i := i
				end
				i := i + 1
			end
			i := best_i
			Result := best_i / 100 / (102 - 1 + (1 + (plane_x - rt.item (i).pos.x_value).abs) * (1 + (plane_y - rt.item (i).pos.y_value).abs) * (1 + (plane_z - rt.item (i).pos.z_value).abs) * (1 + (plane_vx - rt.item (i).vel.x_value).abs) * (1 + (plane_vy - rt.item (i).vel.y_value).abs) * (1 + (plane_vz - rt.item (i).vel.z_value).abs))
			if engine_fire then
				Result := Result / 10
			end
		end

feature -- Checkers
	c1: BOOLEAN is
		do
			Result := (current.nav1_freq - 11175).abs < 1
		end
	c2: BOOLEAN is
		do
			Result := (current.nav1_freq - 11130).abs < 1
		end
	c3: BOOLEAN is
		do
			Result := (data.item (99).item (1) - 11).abs < 1
		end
	c4: BOOLEAN is
		do
			Result := (data.item (104).item (2) - 2).abs < 1
		end
	c5: BOOLEAN is
		do
			Result := not ((data.item (111).item (2) < 1.5 or data.item (111).item (2) > 2.5) and data.item (111).item (6) < 0.5)
		end
	c6: BOOLEAN is
		do
			Result := not (data.item (111).item (2) < 0.5 or data.item (111).item (2) > 1.5)
		end
	c7: BOOLEAN is
		do
			Result := not ((data.item (111).item (3) < 8.5 or data.item (111).item (3) > 9.5) and data.item (111).item (7) < 0.5)
		end
	c8: BOOLEAN is
		do
			Result := (data.item (111).item (3) - 7).abs < 1
		end
	c9: BOOLEAN is
		do
			Result := (data.item (111).item (5) - 1).abs < 0.5
		end
	c10: BOOLEAN is
		do
			Result := (data.item (111).item (3) - 4).abs < 1
		end
	c11: BOOLEAN is
		do
			Result := wheel_brake
		end
	c12: BOOLEAN is
		do
			Result := (current.throttle - 1).abs < 0.3
		end
	c13: BOOLEAN is
		do
			Result := (current.throttle - 0.5).abs < 0.3
		end
	c14: BOOLEAN is
		do
			Result := (current.throttle - 0).abs < 0.3
		end
	c15: BOOLEAN is
		do
			Result := (current.flap_handle - 0).abs < 0.25
		end
	c16: BOOLEAN is
		do
			Result := (current.flap_handle - 0.3).abs < 0.25
		end
	c17: BOOLEAN is
		do
			Result := (current.flap_handle - 1).abs < 0.25
		end
	c18: BOOLEAN is
		do
			Result := current.gear
		end
	c19: BOOLEAN is
		do
			Result := (data.item (103).item (4) - 0).abs < 0.5
		end
	c20: BOOLEAN is
		do
			Result := (data.item (103).item (4) - 2).abs < 0.5
		end
	c21: BOOLEAN is
		do
			Result := ((data.item (112).item (4) - 700).abs < 2) and ((data.item (112).item (3) - 100).abs < 2)
		end
	c22: BOOLEAN is
		do
			Result := (data.item (104).item (4) / 30 - current.joystick_rudder).abs < 0.05
		end
	c23: BOOLEAN is
		do
			Result := (0 - current.joystick_rudder).abs < 0.05
		end
	chkf (i: INTEGER): BOOLEAN is
		do
			inspect i
			when 1 then
				Result := c1
			when 2 then
				Result := c2
			when 3 then
				Result := c3
			when 4 then
				Result := c4
			when 5 then
				Result := c5
			when 6 then
				Result := c6
			when 7 then
				Result := c7
			when 8 then
				Result := c8
			when 9 then
				Result := c9
			when 10 then
				Result := c10
			when 11 then
				Result := c11
			when 12 then
				Result := c12
			when 13 then
				Result := c13
			when 14 then
				Result := c14
			when 15 then
				Result := c15
			when 16 then
				Result := c16
			when 17 then
				Result := c17
			when 18 then
				Result := c18
			when 19 then
				Result := c19
			when 20 then
				Result := c20
			when 21 then
				Result := c21
			when 22 then
				Result := c22
			when 23 then
				Result := c23
			end
		end
	chkmod (i: INTEGER): REAL is
		local
			j, v1, v2: INTEGER
		do
			Result := 0
			if i >= 1 and i <= chk1.height then
				from j := 1
				until j > 23
				loop
					v1 := chk1.item (i, j)
					if v1 > -1 then
						if chkf (j) then
							v2 := 1
						else
							v2 := 0
						end
						if v1 /= v2 then
							Result := Result + 1
						end
					end
					j := j + 1
				end
			end
		end

feature -- Basic operation
	set_rt (rtn: ARRAY [PLANE_POSITION]) is
		do
			rt := rtn
		end
	set_chk (chkn: ARRAY2 [REAL] chkcntn: ARRAY [INTEGER]) is
		do
			chk := chkn
			chkcnt := chkcntn
		end

	make (port_in: INTEGER ip_out: STRING) is
			-- Create using given port for incoming data and port 49000 on given ip for outgoing
		local
			i: INTEGER
		do
			port := port_in
			ip := ip_out
			create data.make (0, 125)
			create data2.make (0, 125)
			from
				i := data.lower
			until
				i > data.upper
			loop
				data.put (create {ARRAY [REAL]}.make (1, 8), i)
				data2.put (create {ARRAY [REAL]}.make (1, 8), i)
				i := i + 1
			end
			create data_types_to_send.make (1, 4)
			data_types_to_send.put (7, 1)
			data_types_to_send.put (11, 2)
			data_types_to_send.put (12, 3)
			data_types_to_send.put (23, 4)
		end

	soc_in: NETWORK_DATAGRAM_SOCKET is
		once
			create Result.make_bound (port)
		end

	soc_out: NETWORK_DATAGRAM_SOCKET is
		once
			create Result.make_targeted_to_ip (ip, 49000)
		end

	receive is
			-- Receive data from X-Plane
		local
			pack: DATAGRAM_PACKET
			cur_data_type_ind, cur_data_type, i, j, bi: INTEGER
			time_step, bd: REAL
		do
			prev_speed := airspeed_knots
			prev_hdef1 := prev_prev_hdef1
			if prev_time = 0 then
				time_step := 0
			else
				time_step := data.item (1).item (1) - prev_time
			end
			prev_time := data.item (1).item (1)
			pack := soc_in.received (36 * data_types_to_receive + 1, 0)
			prev_pack := pack
			from
				cur_data_type_ind := 1
			until
				cur_data_type_ind > data_types_to_receive
			loop
				cur_data_type := data_type (cur_data_type_ind, pack)
				if cur_data_type >= data.lower and cur_data_type <= data.upper then
					from
						i := 1
					until
						i > 8
					loop
						data [cur_data_type].put (read_real (i, cur_data_type_ind, pack), i)
						i := i + 1
					end
				end
				cur_data_type_ind := cur_data_type_ind + 1
			end
			from
				cur_data_type_ind := data.lower
			until
				cur_data_type_ind > data.upper
			loop
				from
					i := 1
				until
					i > 8
				loop
					data2.item (cur_data_type_ind).put (data.item (cur_data_type_ind).item (i), i)
					i := i + 1
				end
				cur_data_type_ind := cur_data_type_ind + 1
			end
			prev_prev_hdef1 := vor1_hdef

			if prev_time1 = 0 then
				prev_time1 := data.item (1).item (1)
			end
			i := (data.item (1).item (1) - prev_time1).ceiling
			if i >= rt1.lower and i <= rt1.upper then
				defl_int := defl_int + ((plane_x - rt1.item (i).pos.x_value).abs + (plane_y - rt1.item (i).pos.y_value).abs + (plane_z - rt1.item (i).pos.z_value).abs + (plane_vx - rt1.item (i).vel.x_value).abs + (plane_vy - rt1.item (i).vel.y_value).abs + (plane_vz - rt1.item (i).vel.z_value).abs + chkmod (i) * 50) * time_step
				time_int := time_int + time_step
			end

			if i >= best_fit_pos.lower and i <= best_fit_pos.upper then
				bi := i
				bd := ((plane_x - rt1.item (i).pos.x_value)^2 + (plane_y - rt1.item (i).pos.y_value)^2 + (plane_z - rt1.item (i).pos.z_value)^2)^0.5
				from j := rt1.lower
				until j > rt1.upper
				loop
					if ((plane_x - rt1.item (j).pos.x_value)^2 + (plane_y - rt1.item (j).pos.y_value)^2 + (plane_z - rt1.item (j).pos.z_value)^2)^0.5 < bd then
						bd := ((plane_x - rt1.item (j).pos.x_value)^2 + (plane_y - rt1.item (j).pos.y_value)^2 + (plane_z - rt1.item (j).pos.z_value)^2)^0.5
						bi := j
					end
					j := j + 1
				end
				best_fit_pos.item (i) := best_fit_pos.item (i) + ((plane_x - rt1.item (bi).pos.x_value)^2 + (plane_y - rt1.item (bi).pos.y_value)^2 + (plane_z - rt1.item (bi).pos.z_value)^2)^0.5
				best_fit_vel.item (i) := best_fit_vel.item (i) + ((plane_vx)^2 + (plane_vy)^2 + (plane_vz)^2)^0.5 - ((rt1.item (bi).vel.x_value)^2 + (rt1.item (bi).vel.y_value)^2 + (rt1.item (bi).vel.z_value)^2)^0.5
				best_fit_alt.item (i) := best_fit_alt.item (i) + plane_y
				best_fit_gload.item (i) := best_fit_gload.item (i) + data.item (3).item (2)
				best_fit_cnt.item (i) := best_fit_cnt.item (i) + 1
			end
		end

	send is
			-- Send data to X_Plane
		local
			pack: DATAGRAM_PACKET
			cur_data_type_ind, cur_data_type, i: INTEGER
		do
			pack := prev_pack
			from
				cur_data_type_ind := data.lower
			until
				cur_data_type_ind > data.upper
			loop
				from
					i := 1
				until
					i > 8
				loop
					if data.item (cur_data_type_ind).item (i) = data2.item (cur_data_type_ind).item (i) then
						data.item (cur_data_type_ind).put (-999, i)
					end
					i := i + 1
				end
				cur_data_type_ind := cur_data_type_ind + 1
			end
			from
				cur_data_type_ind := 1
			until
				cur_data_type_ind > data_types_to_receive
			loop
				cur_data_type := data_type (cur_data_type_ind, pack)
				from
					i := 1
				until
					i > 8
				loop
					put_real (data.item (cur_data_type).item (i), i, cur_data_type_ind, pack)
					i := i + 1
				end
				cur_data_type_ind := cur_data_type_ind + 1
			end
			soc_out.send (pack, 0)
		end

	send2 is
			-- Send data to X_Plane
		local
			pack: DATAGRAM_PACKET
			cur_data_type_ind, cur_data_type, i, j: INTEGER
		do
			create pack.make (data_types_to_send.upper * 9 * 4 + 1)
			pack.put_element ('D', -4)
			pack.put_element ('A', -3)
			pack.put_element ('T', -2)
			pack.put_element ('A', -1)
			i := 0
			pack.put_element (i.to_character_8, 0)
			from
				cur_data_type_ind := data_types_to_send.lower
				j := 1
			until
				cur_data_type_ind > data_types_to_send.upper
			loop
				cur_data_type := data_types_to_send.item (cur_data_type_ind)
				i := 0
				pack.put_element (i.to_character_8, j)
				j := j + 1
				pack.put_element (i.to_character_8, j)
				j := j + 1
				pack.put_element (i.to_character_8, j)
				j := j + 1
				pack.put_element (cur_data_type.to_character_8, j)
				j := j + 1
				from
					i := 1
				until
					i > 8
				loop
					if data.item (cur_data_type).item (i) = data2.item (cur_data_type).item (i) then
						data.item (cur_data_type).put (-999, i)
					end
					i := i + 1
				end
				cur_data_type_ind := cur_data_type_ind + 1
			end
			from
				cur_data_type_ind := 1
			until
				cur_data_type_ind > data_types_to_receive
			loop
				cur_data_type := data_type (cur_data_type_ind, pack)
				from
					i := 1
				until
					i > 8
				loop
					put_real (data.item (cur_data_type).item (i), i, cur_data_type_ind, pack)
					i := i + 1
				end
				cur_data_type_ind := cur_data_type_ind + 1
			end
			soc_out.send (pack, 0)
		end

	print_data is
			-- Print data to io
		do
			io.put_string (" Plane: ")
			io.put_string ("Position: ")
			io.put_string ("X: ")
			io.put_real (plane_x)
			io.put_string (" Y: ")
			io.put_real (plane_y)
			io.put_string (" Z: ")
			io.put_real (plane_z)
			io.put_string (" Speed: ")
			io.put_string ("X: ")
			io.put_real (plane_vx)
			io.put_string (" Y: ")
			io.put_real (plane_vy)
			io.put_string (" Z: ")
			io.put_real (plane_vz)
			io.put_string (" Distance: ")
			io.put_real (distance_ft)
			io.put_string ("ft = ")
			io.put_real (distance_nm)
			io.put_string ("nm")

			io.put_string (" Engine RPM: ")
			io.put_real (engine_rpm)
			io.put_string (" Ground location: ")
			io.put_string ("Centre: ")
			io.put_string ("X: ")
			io.put_real (ground_location_x)
			io.put_string (" Y: ")
			io.put_real (ground_location_y)
			io.put_string (" Z: ")
			io.put_real (ground_location_z)
			io.put_string (" Slope: ")
			io.put_string ("X: ")
			io.put_real (ground_location_slope_x)
			io.put_string (" Z: ")
			io.put_real (ground_location_slope_z)
			io.put_new_line
		end

feature -- Status reporting
	elapsed_time: REAL is
		do
			receive
			Result := data.item (1).item (1)
		end

	airspeed_knots: REAL is
		do
			Result := data.item (2).item (2)
		end

	joystick_x: REAL is
		do
			Result := data.item (7).item (2)
		end
	joystick_y: REAL is
		do
			Result := data.item (7).item (1)
		end
	joystick_rudder: REAL is
		do
			Result := data.item (7).item (3)
		end
	flap_handle: REAL is
		do
			Result := data.item (11).item (5)
		end
	flap_position: REAL is
		do
			Result := data.item (11).item (6)
		end
	speed_brakes: REAL is
		do
			Result := data.item (11).item (8)
		end
	gear: BOOLEAN is
		do
			Result := data.item (12).item (1) > 0.5
		end
	wheel_brake: BOOLEAN is
		do
			Result := data.item (12).item (2) > 0.5
		end

	d_pitch: REAL is
		do
			Result := data.item (15).item (1)
		end
	d_roll: REAL is
		do
			Result := data.item (15).item (2)
		end
	d_rud: REAL is
		do
			Result := data.item (15).item (3)
		end

	pitch: REAL is
		do
			Result := data.item (16).item (1)
		end
	roll: REAL is
		do
			Result := data.item (16).item (2)
		end

	altitude_above_sea_level_ft: REAL is
		do
			Result := data.item (18).item (3)
		end
	altitude_above_ground_ft: REAL is
		do
			Result := data.item (18).item (4)
		end

	plane_x: REAL is
		do
			Result := data.item (19).item (1)
		end
	plane_y: REAL is
		do
			Result := data.item (19).item (2)
		end
	plane_z: REAL is
		do
			Result := data.item (19).item (3)
		end
	plane_vx: REAL is
		do
			Result := data.item (19).item (4)
		end
	plane_vy: REAL is
		do
			Result := data.item (19).item (5)
		end
	plane_vz: REAL is
		do
			Result := data.item (19).item (6)
		end
	distance_ft: REAL is
		do
			Result := data.item (19).item (7)
		end
	distance_nm: REAL is
		do
			Result := data.item (19).item (8)
		end

	throttle: REAL is
		do
			Result := data.item (23).item (1)
		end

	engine_rpm: REAL is
		do
			Result := data.item (34).item (1)
		end

	nav1_freq: REAL is
		do
			Result := data.item (94).item (1)
		end

	vor1_hdef: REAL is
		do
			Result := data.item (96).item (1)
		end
	vor1_vdef: REAL is
		do
			Result := data.item (96).item (2)
		end
	vor1_brg: REAL is
		do
			Result := data.item (96).item (3)
		end
	vor1_dist: REAL is
		do
			Result := data.item (96).item (4)
		end
	vor2_hdef: REAL is
		do
			Result := data.item (96).item (5)
		end
	vor2_vdef: REAL is
		do
			Result := data.item (96).item (6)
		end
	vor2_brg: REAL is
		do
			Result := data.item (96).item (7)
		end
	vor2_dist: REAL is
		do
			Result := data.item (96).item (8)
		end
	d_vor1_hdef: REAL is
		do
			Result := (vor1_hdef - prev_hdef1) / (data.item (1).item (1) - prev_time)
		end
	gps_dist: REAL is
		do
			Result := data.item (99).item (3)
		end

	engine_fire: BOOLEAN is
		do
			Result := data.item (110).item (7) > 10
		end

	ground_location_x: REAL is
		do
			Result := data.item (125).item (1)
		end
	ground_location_y: REAL is
		do
			Result := data.item (125).item (2)
		end
	ground_location_z: REAL is
		do
			Result := data.item (125).item (3)
		end
	ground_location_slope_x: REAL is
		do
			Result := data.item (125).item (4)
		end
	ground_location_slope_z: REAL is
		do
			Result := data.item (125).item (5)
		end

	acceleration: REAL is
		do
			Result := (airspeed_knots - prev_speed) / (data.item (1).item (1) - prev_time)
		end

feature -- Status setting
	set_joystick_x (level: REAL) is
		do
			data.item (7).put (level, 2)
		end
	set_joystick_y (level: REAL) is
		do
			data.item (7).put (level, 1)
		end
	set_joystick_rudder (level: REAL) is
		do
			data.item (7).put (level, 3)
		end
	set_flaps (level: REAL) is
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > 8
			loop
				data.item (11).put (-999, i)
				i := i + 1
			end
			data.item (11).put (level, 5)
			send_one (11)
		end
	set_speed_brakes (level: REAL) is
		do
			data.item (11).put (level, 8)
		end

	set_gear (down: BOOLEAN) is
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > 8
			loop
				data.item (12).put (-999, i)
				i := i + 1
			end
			if down then
				data.item (12).put (1, 1)
			else
				data.item (12).put (0, 1)
			end
			send_one (12)
		end
	set_wheel_brake (on: BOOLEAN) is
		do
			if on then
				data.item (12).put (1, 2)
			else
				data.item (12).put (0, 2)
			end
			data2.item (12).put (0.5, 2)
		end

	set_throttle (level: REAL) is
		local
			i: INTEGER
		do
			receive
			from
				i := 1
			until
				i > 8
			loop
				data.item (23).put (-999, i)
				i := i + 1
			end
			data.item (23).put (level, 1)
			data.item (23).put (level, 2)
			data.item (23).put (level, 3)
			data.item (23).put (level, 4)
			send_one (23)
		end

	leave_nav_freq_alone is
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > 8
			loop
				data.item (94).put (-999, i)
				data.item (96).put (-999, i)
				i := i + 1
			end
		end

	set_fdir_auto is
		do
			send_char ('q')
			receive
			send_char ('q')
		end
	set_autopilot_localizer is
		do
			receive
			receive
			if (data.item (111).item (2) < 1.5 or data.item (111).item (2) > 2.5) and data.item (111).item (6) < 0.5 then
				send_char ('%%')
			end
			receive
			receive
			if (data.item (111).item (2) < 1.5 or data.item (111).item (2) > 2.5) and data.item (111).item (6) < 0.5 then
				send_char ('%%')
			end
		end
	set_autopilot_back_course is
		do
			send_char ('w')
		end

	half_fill_with_fuel is
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > 8
			loop
				data.item (59).put (-999, i)
				i := i + 1
			end
			data.item (59).put (183, 1)
			data.item (59).put (183, 2)
			send_one (59)
		end

	set_gps_mode_waypoint is
		do
			send_char ('t')
		end
	set_nav1_freq (freq: REAL) is
		local
			i: INTEGER
		do
			i := (freq * 100).floor
			from
				receive
			until
				data.item (94).item (1).floor \\ 100 = i \\ 100
			loop
				send_char ('i')
				receive
				receive
			end
			from
			until
				data.item (94).item (1).floor // 100 = i // 100
			loop
				send_char ('u')
				receive
				receive
			end
		end


	load_aircraft (path: STRING) is
			-- Load plane
		local
			pack: DATAGRAM_PACKET
			i, j: INTEGER
		do
			create pack.make (1116)
			pack.put_element ('V', -4)
			pack.put_element ('E', -3)
			pack.put_element ('H', -2)
			pack.put_element ('N', -1)
			j := 17
			pack.put_element (j.to_character_8, 0)

			j := 0
			pack.put_element (j.to_character_8, 1)
			pack.put_element (j.to_character_8, 2)
			pack.put_element (j.to_character_8, 3)
			pack.put_element (j.to_character_8, 4)

			from i := 1
			until i > path.count
			loop
				pack.put_element (path.item (i), i + 4)
				i := i + 1
			end
			from
			until i > 152 + 24 * 40
			loop
				pack.put_element (j.to_character_8, i + 4)
				i := i + 1
			end

			soc_out.send (pack, 0)
		end

	set_to_airport (apt: STRING rwy: INTEGER reverse: BOOLEAN) is
			-- Set plane to given airport
		local
			pack: DATAGRAM_PACKET
			i: INTEGER
		do
			create pack.make (21)
			pack.put_element ('P', -4)
			pack.put_element ('A', -3)
			pack.put_element ('P', -2)
			pack.put_element ('T', -1)
			i := 0
			pack.put_element (i.to_character_8, 0)
			pack.put_element (apt.item (1), 1)
			pack.put_element (apt.item (2), 2)
			pack.put_element (apt.item (3), 3)
			pack.put_element (apt.item (4), 4)

			pack.put_element (i.to_character_8, 5)
			pack.put_element (i.to_character_8, 6)
			pack.put_element (i.to_character_8, 7)
			pack.put_element (i.to_character_8, 8)

			pack.put_element (i.to_character_8, 9)
			pack.put_element (i.to_character_8, 10)
			i := 2
			pack.put_element (i.to_character_8, 11)
			i := 89
			pack.put_element (i.to_character_8, 12)

			i := 0
			pack.put_element (i.to_character_8, 13)
			pack.put_element (i.to_character_8, 14)
			pack.put_element (i.to_character_8, 15)
			pack.put_element (rwy.to_character_8, 16)

			pack.put_element (i.to_character_8, 17)
			pack.put_element (i.to_character_8, 18)
			pack.put_element (i.to_character_8, 19)
			if reverse then
				i := 1
			else
				i := 0
			end
			pack.put_element (i.to_character_8, 20)

			soc_out.send (pack, 0)
		end

	send_char (c: CHARACTER_8) is
			-- Send keypress to XPlane
		local
			pack: DATAGRAM_PACKET
			i: INTEGER
		do
			create pack.make (5)
			pack.put_element ('C', -4)
			pack.put_element ('H', -3)
			pack.put_element ('A', -2)
			pack.put_element ('R', -1)
			i := 0
			pack.put_element (i.to_character_8, 0)
			pack.put_element (i.to_character_8, 1)
			pack.put_element (i.to_character_8, 2)
			pack.put_element (i.to_character_8, 3)
			pack.put_element (c, 4)
			soc_out.send (pack, 0)
		end

	send_one (ind: INTEGER) is
		local
			pack: DATAGRAM_PACKET
			i: INTEGER
		do
			create pack.make (37)
			pack.put_element ('D', -4)
			pack.put_element ('A', -3)
			pack.put_element ('T', -2)
			pack.put_element ('A', -1)
			i := 0
			pack.put_element (i.to_character_8, 0)

			pack.put_element (i.to_character_8, 1)
			pack.put_element (i.to_character_8, 2)
			pack.put_element (i.to_character_8, 3)
			pack.put_element (ind.to_character_8, 4)

			from
				i := 1
			until
				i > 8
			loop
				put_real (data.item (ind).item (i), i, 1, pack)
				i := i + 1
			end

			soc_out.send (pack, 0)
		end
	save_best_fit is
		local
			f: PLAIN_TEXT_FILE
			i: INTEGER
		do
			create f.make_open_write ("best_fit.txt")
			from i := 1
			until i > 300
			loop
				if best_fit_cnt.item (i) > 0 then
					f.put_real (best_fit.item (i) / best_fit_cnt.item (i))
				else
					f.put_real (best_fit.item (i))
				end
				f.put_new_line
				i := i + 1
			end
			f.close
			prev_time1 := 0
		end
	save_best_fit_pos_vel is
		local
			f: PLAIN_TEXT_FILE
			i: INTEGER
		do
			create f.make_open_write ("best_fit_pos.txt")
			from i := 1
			until i > 300
			loop
				if best_fit_cnt.item (i) > 0 then
					f.put_real (best_fit_pos.item (i) / best_fit_cnt.item (i))
				else
					f.put_real (best_fit_pos.item (i))
				end
				f.put_new_line
				i := i + 1
			end
			f.close
			create f.make_open_write ("best_fit_vel.txt")
			from i := 1
			until i > 300
			loop
				if best_fit_cnt.item (i) > 0 then
					f.put_real (best_fit_vel.item (i) / best_fit_cnt.item (i))
				else
					f.put_real (best_fit_vel.item (i))
				end
				f.put_new_line
				i := i + 1
			end
			f.close
			create f.make_open_write ("best_fit_alt.txt")
			from i := 1
			until i > 300
			loop
				if best_fit_cnt.item (i) > 0 then
					f.put_real (best_fit_alt.item (i) / best_fit_cnt.item (i))
				else
					f.put_real (best_fit_alt.item (i))
				end
				f.put_new_line
				i := i + 1
			end
			f.close
			create f.make_open_write ("best_fit_gload.txt")
			from i := 1
			until i > 300
			loop
				if best_fit_cnt.item (i) > 0 then
					f.put_real (best_fit_gload.item (i) / best_fit_cnt.item (i))
				else
					f.put_real (best_fit_gload.item (i))
				end
				f.put_new_line
				i := i + 1
			end
			f.close
		end
	set_prev_time1_to_zero is
		do
			prev_time1 := 0
		end

feature {NONE} -- Implementation
	reverse_bytes_to_real (b1, b2, b3, b4: CHARACTER_8): REAL_32 is
			-- Converts macintosh byte representation to real
		external
			"C signature (byte, byte, byte, byte): EIF_REAL_32 use %"conv.h%""
		end

	read_real (i, cur_data_type_ind: INTEGER pack: DATAGRAM_PACKET): REAL_32 is
			-- Returns real with given index after prologue and integer in data packet
		local
			j: INTEGER
		do
			j := cur_data_type_ind * 36 - 36 + i * 4 + 1
			Result := reverse_bytes_to_real (pack.element (j), pack.element (j + 1), pack.element (j + 2), pack.element (j + 3))
		end

	data_type (cur_data_type_ind: INTEGER pack: DATAGRAM_PACKET): INTEGER is
			-- Returns type of data packet
		do
			Result := pack.element (cur_data_type_ind * 36 - 36 + 4).code
		end

	print_pack (pack: DATAGRAM_PACKET) is
			-- Print packet bytes to io
		local
			i: INTEGER
		do
			io.put_new_line
			from i := 0 until i >= pack.count loop
				io.put_integer (pack.element (i).code)
				io.put_string (" ")
				i := i + 1
			end
			io.put_new_line
		end

	real_to_byte (r: REAL_32 i: INTEGER): CHARACTER_8 is
			-- Returns i-th byte of r, i is counted from 0
		external
			"C signature (EIF_REAL_32, int): byte use %"conv.h%""
		end

	put_real (r: REAL_32 i, cur_data_type_ind: INTEGER pack: DATAGRAM_PACKET) is
			-- Puts real into given index after prologue and integer in data packet
		local
			j: INTEGER
		do
			j := cur_data_type_ind * 36 - 36 + i * 4 + 1
			pack.put_element (real_to_byte (r, 3), j)
			pack.put_element (real_to_byte (r, 2), j + 1)
			pack.put_element (real_to_byte (r, 1), j + 2)
			pack.put_element (real_to_byte (r, 0), j + 3)
		end

	port: INTEGER
	ip: STRING
	prev_pack: DATAGRAM_PACKET
	data, data2: ARRAY [ARRAY [REAL]]
	prev_speed, prev_hdef1, prev_prev_hdef1, prev_time: REAL
	rt: ARRAY [PLANE_POSITION]
	rt1: ARRAY [PLANE_POSITION] is
		local
			i: INTEGER
			f: PLAIN_TEXT_FILE
		once
			create Result.make (1, 300)
			create f.make_open_read ("route.txt")
			from i := 1
			until i > 300
			loop
				Result.put (create {PLANE_POSITION}.make, i)
				Result.item (i).load_from_file (f)
				i := i + 1
			end
			f.close
		end
	chk: ARRAY2 [REAL]
	chkcnt: ARRAY [INTEGER]
	chk1: ARRAY2 [INTEGER] is
		local
			i, j: INTEGER
			f: PLAIN_TEXT_FILE
		once
			create Result.make (300, 23)
			create f.make_open_read ("chk.txt")
			from i := 1
			until i > 300
			loop
				from j := 1
				until j > 23
				loop
					f.read_integer
					Result.put (f.last_integer, i, j)
					j := j + 1
				end
				i := i + 1
			end
			f.close
		end
	best_fit: ARRAY [REAL] is
		once
			create Result.make (1, 300)
		end
	best_fit_cnt: ARRAY [INTEGER] is
		once
			create Result.make (1, 300)
		end
	best_fit_pos: ARRAY [REAL] is
		once
			create Result.make (1, 300)
		end
	best_fit_vel: ARRAY [REAL] is
		once
			create Result.make (1, 300)
		end
	best_fit_alt: ARRAY [REAL] is
		once
			create Result.make (1, 300)
		end
	best_fit_gload: ARRAY [REAL] is
		once
			create Result.make (1, 300)
		end
	prev_time1, defl_int, time_int: REAL

feature {NONE} -- Data
	data_types_to_send: ARRAY [INTEGER]
	data_types_to_receive: INTEGER is 19 -- 1, 2, 7, 11, 12, 15, 16, 18, 19, 23, 34, 59, 94, 99, 104, 110, 111, 112

end
