indexing
	description: ""
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	XPLANE_CO

inherit
	GENETIC_CONTROL_OBJECT
	redefine
		timeout,
		show_computational_state
	end

	REAL_COMPARISON
	export {NONE}
		all
	end

create
	make

feature -- Initialization
	make is
			-- Create control object
		do
			set_step_duration(default_step_duration)
			brk.init
			prev_time := brk.elapsed_time
			if timeout_sec = 0 then
				timeout_sec := 10
			end
		ensure
			step_duration_default: step_duration = default_step_duration
		end

feature -- Access
	Moving: INTEGER is 1
	Can_fly_with_flaps: INTEGER is 2
	Can_fly_without_flaps: INTEGER is 3
	Flying: INTEGER is 4
	Near_surface: INTEGER is 5
	On_echelon_altitude: INTEGER is 6
	Reaching_gps_target: INTEGER is 7
			-- Predicate IDs

	predicate_count: INTEGER is 7
			-- Number of predicates	

	Set_departure_nav_freq: INTEGER is 1
	Set_arrival_nav_freq: INTEGER is 2
	Set_gps_to_waypoint_mode: INTEGER is 3
	Set_flight_director_to_auto: INTEGER is 4
	Set_autopilot_localizer_mode: INTEGER is 5
	Set_autopilot_heading_hold: INTEGER is 6
	Set_autopilot_glide_slope: INTEGER is 7
	Set_autopilot_altitude_hold: INTEGER is 8
	Set_autopilot_vertical_speed_hold: INTEGER is 9
	Set_wheel_brake_on: INTEGER is 10
	Set_wheel_brake_off: INTEGER is 11
	Set_throttle_full: INTEGER is 12
	Set_throttle_half: INTEGER is 13
	Set_throttle_min: INTEGER is 14
	Set_no_flaps: INTEGER is 15
	Set_takeoff_flaps: INTEGER is 16
	Set_landing_flaps: INTEGER is 17
	Set_gear_down: INTEGER is 18
	Set_gear_up: INTEGER is 19
	Set_hsi_to_nav1: INTEGER is 20
	Set_hsi_to_gps: INTEGER is 21
	Set_vertical_speed_and_altitude: INTEGER is 22
	Translate_autopilot_to_rudder: INTEGER is 23
	Set_rudder_centered: INTEGER is 24
			-- Action IDs

	action_count: INTEGER is 24
			-- Number of actions	

feature -- Measurement
	fitness: REAL is
			-- Calculate fitness
		do
			Result := brk.fitness
		end
	timeout: BOOLEAN is
		do
			Result := brk.elapsed_time - prev_time > timeout_sec
		end
	update_timeout (fit: REAL) is
		do
			if timeout_sec < 300 and fit > 0.1 * best_fit.item (timeout_sec) then
				timeout_sec := timeout_sec + 2
			end
		end
	set_timeout (t: INTEGER) is
		do
			timeout_sec := t
		end
	timeout_sec: INTEGER

feature -- Miscellaneous
	notify_of_new_generation is
		do
			make
		end

feature -- Output
	show_computational_state is
			-- Print computational state of `Current'
		do
			print ("distance: "); io.put_real (brk.distance_nm); io.new_line
			print ("x: "); io.put_real (brk.plane_x); io.new_line
			print ("y: "); io.put_real (brk.plane_y); io.new_line
			print ("z: "); io.put_real (brk.plane_z); io.new_line
			print ("vx: "); io.put_real (brk.plane_vx); io.new_line
			print ("vy: "); io.put_real (brk.plane_vy); io.new_line
			print ("vz: "); io.put_real (brk.plane_vz); io.new_line
			io.new_line
		end
	brk: X_PLANE_BROKER is
		once
			create Result.make (48999, "127.0.0.1")
		end

feature {NONE} -- Implementation
	prev_time: REAL

	x (i: INTEGER): BOOLEAN is
			-- Value of predicate with id `i'
		do
			inspect i
			when Moving then Result := brk.moving
			when Can_fly_with_flaps then Result := brk.can_fly_with_flaps
			when Can_fly_without_flaps then Result := brk.can_fly_without_flaps
			when Flying then Result := brk.flying
			when Near_surface then Result := brk.near_surface
			when On_echelon_altitude then Result := brk.on_echelon_altitude
			when Reaching_gps_target then Result := brk.reaching_gps_target
			end
		end

	z (i: INTEGER) is
			-- Perform action with id `i'
		do
			inspect i
			when Set_departure_nav_freq then brk.set_departure_nav_freq
			when Set_arrival_nav_freq then brk.set_arrival_nav_freq
			when Set_gps_to_waypoint_mode then brk.set_gps_to_waypoint_mode
			when Set_flight_director_to_auto then brk.set_flight_director_to_auto
			when Set_autopilot_localizer_mode then brk.set_autopilot_localizer_mode
			when Set_autopilot_heading_hold then brk.set_autopilot_heading_hold
			when Set_autopilot_glide_slope then brk.set_autopilot_glide_slope
			when Set_autopilot_altitude_hold then brk.set_autopilot_altitude_hold
			when Set_autopilot_vertical_speed_hold then brk.set_autopilot_vertical_speed_hold
			when Set_wheel_brake_on then brk.set_wheel_brake_on
			when Set_wheel_brake_off then brk.set_wheel_brake_off
			when Set_throttle_full then brk.set_throttle_full
			when Set_throttle_half then brk.set_throttle_half
			when Set_throttle_min then brk.set_throttle_min
			when Set_no_flaps then brk.set_no_flaps
			when Set_takeoff_flaps then brk.set_takeoff_flaps
			when Set_landing_flaps then brk.set_landing_flaps
			when Set_gear_down then brk.set_gear_down
			when Set_gear_up then brk.set_gear_up
			when Set_hsi_to_nav1 then brk.set_hsi_to_nav1
			when Set_hsi_to_gps then brk.set_hsi_to_gps
			when Set_vertical_speed_and_altitude then brk.set_vertical_speed_and_altitude
			when Translate_autopilot_to_rudder then brk.translate_autopilot_to_rudder
			when Set_rudder_centered then brk.set_rudder_centered
			end
		end

	actual_restart is
			-- Bring `Current' to initial state, but do not randomize
		do
			brk.init
			prev_time := brk.elapsed_time
		end

	actual_do_step is
		do
			brk.receive
		end

	default_step_count: INTEGER is 6000
			-- Proper step count	

	r: MY_RANDOM is
			-- Random generator for implementation needs
		once
			create Result.make
		end

	n_preds: INTEGER is 7

	best_fit: ARRAY [REAL] is
		local
			i: INTEGER
			f: PLAIN_TEXT_FILE
		once
			create Result.make (1, 300)
			create f.make_open_read ("best_fit.txt")
			from i := 1
			until i > 300
			loop
				f.read_real
				Result.put (f.last_real, i)
				i := i + 1
			end
			f.close
		end

invariant
	r_exists: r /= Void
end
